#!/usr/bin/perl
#
# Watch a directory for video files and convert.
# (c) Andrew Flegg 2006. Released under the Artistic Licence.

use warnings;
use strict;
use LWP::Simple;
use LWP::UserAgent;
use vars qw($INCOMING_DIR $OUTGOING_DIR $MAX_SIZE);

$INCOMING_DIR = '/home/andrew/lib/www/vidconvert';
$OUTGOING_DIR = '/usr/local/share/www/bleb/services/vidconvert/results';
$MAX_SIZE     = 42 * 1024 * 1024;

chdir($INCOMING_DIR);
while(1) {
   sleep 5;
   my $next = `/bin/ls -rt | head -n 1`;
   chomp($next);

   next unless $next;
   next if $next =~ /\.lock$/ or -f "$next.lock";

   my ($key) = $next =~ /(\w+)/;
   warn "+++ ".localtime()." - Got $next for $key\n";

   if ($next =~ /\.url$/) {
      my $url = `/bin/cat $INCOMING_DIR/\Q$next\E`; chomp($url);
      if ($url =~ m!youtube\.com/watch!) {
         warn "Youtube-dl of $url\n";
         my $data = get($url) || '';

         my ($flv) = $data =~ /player2\.swf\?(video_id=.*?&t=[^&\"]+)/i;
         if ($flv) {
             system("echo Downloading... 2>>$OUTGOING_DIR/$key.txt 1>&2");

             my $ua = LWP::UserAgent->new();
             $ua->max_size( $MAX_SIZE );
             $ua->get( "http://www.youtube.com/get_video?$flv",
                       ':content_file' => "$key.flv" );
         }

         unlink $next;
         $next = "$key.flv";
         next unless $flv;

      } elsif ($url =~ m!video\.google\..*?/videoplay!) {
         warn "Google download of $url\n";
         my $data = get($url) || '';

         my ($flv) = $data =~ /videoUrl.*?(http.*?)[\&\"]/;
         if ($flv) {
            $flv =~ s/%([0-9A-F]{2})/pack("H*", $1)/ge;
            system("echo Downloading... 2>>$OUTGOING_DIR/$key.txt 1>&2");
            my $ua = LWP::UserAgent->new();
            $ua->max_size( $MAX_SIZE );
            $ua->get( $flv, ':content_file' => "$key.flv" );
         }
         unlink $next;
         $next = "$key.flv";
         next unless $flv;

      } else {
         warn "Download of $url\n";
         my ($ext) = $url =~ /[^?]+\.(\w+)\b/;
         unless ($ext) {
            warn "Skipping $url as there's no extension.\n";
            unlink $next;
            next;
         }
         system("echo Downloading... 2>>$OUTGOING_DIR/$key.txt 1>&2");
         my $ua = LWP::UserAgent->new();
         $ua->max_size( $MAX_SIZE );
         $ua->get( $url, ':content_file' => "$key.$ext" );
         unlink $next;
         $next = "$key.$ext";
      }
   }

   warn "Invoking 770-encode for $next to $key.avi\n";
   system("nice -n 10 770-encode -p smallest $next $OUTGOING_DIR/$key.avi 2>>$OUTGOING_DIR/$key.txt 1>&2");
   warn "Failed to produce $key.avi\n" unless -f "$OUTGOING_DIR/$key.avi";
   unlink($next);

   system("echo 'VIDCONVERT COMPLETE' >>$OUTGOING_DIR/$key.txt");

   sleep 2;
   warn "+++ ".localtime()." - Pausing for next object in queue.\n";
}

exit;
