#!/usr/bin/perl -w

use warnings;
use strict;
use CGI;
use vars qw($OUTGOING_DIR $INCOMING_DIR);

$INCOMING_DIR = '/home/andrew/lib/www/vidconvert';
$OUTGOING_DIR = '/usr/local/share/www/bleb/services/vidconvert/results';

my $queue_size = `ls $INCOMING_DIR | cut -d. -f 1 | uniq | wc -l`;

my $query = new CGI;
my ($key) = ($query->param('key') || '') =~ /(\w+)/; $key ||= 'x';
my $this  = $ENV{'REQUEST_URI'} || '';

my $in_queue = system("ls $INCOMING_DIR/$key.* 1>/dev/null 2>&1") == 0;

print $query->header('text/html');
if (!$key) {
  print <<EOM;
<h1>Invalid key</h1>
<p>No key was specified.</p>
EOM
  exit;

} elsif (! -f "$OUTGOING_DIR/$key.txt" && $in_queue) {
  print <<EOM;
<html>
<head>
<meta http-equiv="refresh" content="10" />
<title>VidConvert: Please wait</title>
</head>
<body>
<h1>Please wait</h1>
<p>Your video is in the queue, there are currently $queue_size items in the
queue. Please <a href="$this">check again</a> shortly; this page should reload
in 10 seconds.</p>
</body>
</html>
EOM
  exit;

} elsif (! -f "$OUTGOING_DIR/$key.txt") {
  print <<EOM;
<h1>Invalid key</h1>
<p>No results were found, and the request is not in the queue. There are
two possible reasons for this:</p>
<ol>
<li>The key in the URL is incorrect.</li>
<li>The submitted video was processed over 24 hours ago and the results
deleted.</li>
</ol>
EOM
  exit;
}

$| = 1;
print <<EOM;
<html>
<head>
<title>VidConvert Results</title>
<link rel="stylesheet" href="/services/vidconvert/styles.css" type="text/css" />
</head>

<body>
<h1>VidConvert Results</h1>

<div id="status"><pre>
EOM

my $complete  = `tail -n 1 $OUTGOING_DIR/$key.txt` =~ /VIDCONVERT COMPLETE/;
my $count     = 0;
my $last_line = '';
open(IN, "<$OUTGOING_DIR/$key.txt") or die "Unable to open $key.txt: $!\n";
seek IN, -640, 2;
while(!$complete) {
  $/ = \80;
  my $line = <IN>;

  unless ($line) {
    sleep 2;
    next;
  }

  $last_line .= $line;
  $last_line  =~ s/.*?[\r\n](.*)/$1/;
  print $line;
  if (++$count % 20 == 0) {
    print <<EOM;
<script type="text/javascript"><!--
window.scrollTo(0, ${count}000);
// --></script>
EOM
  }
  $complete = $last_line =~ /VIDCONVERT COMPLETE/;
}

my $size = `ls -lh $OUTGOING_DIR/$key.avi 2>&1 | cut -d' ' -f 5`;
$size =~ s/[\s\r\n]+$//;

print <<EOM;
</pre></div>
<script type="text/javascript"><!--
  document.getElementById('status').style.display = 'none';
// --></script>

<p>Conversion complete.</p>
EOM

if ($size eq 'file') {
  print <<EOM;
<p>Conversion did not result in video file. Sorry. Perhaps the
<a href="/services/vidconvert/results/$key.txt">log</a> will be useful?</p>

<p>Please note that RealVideo cannot be supported, and other proprietary codecs
may cause problems.</p>
EOM

} else {
  print <<EOM;
<p class="results"><a href="/services/vidconvert/results/$key.avi">$key.avi</a> ($size)</p>
EOM
}

print <<EOM;
<p><a href="/services/vidconvert/">Convert another</a>.</p>

<p class="footer">&copy; <a href="http://www.bleb.org/">Andrew Flegg</a> 2006.</p>
</body>
</html>
EOM

exit;
